#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "profile.h"

static int on_timeout (gpointer data);
static long int time;
static int time_out;
static int profile_no = 0;
static int timeout_runing = 0;

static void event_execute(char *cmd) {
	if (fork() == 0) {
		/* child proces */
		execlp("sh", "sh", "-v", "-c", cmd, NULL);
	}
}


void
on_w_main_destroy                      (GtkObject       *object,
                                        gpointer         user_data)
{
	if (timeout_runing && profile[profile_no].cmd_stop_en)
		event_execute(profile[profile_no].cmd_stop);
	
	gtk_main_quit();
}


void
on_e_profile_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char actual_name[256];
	int i;
	
	strcpy(actual_name, gtk_entry_get_text(GTK_ENTRY(e_profile)));
	actual_name[strlen(actual_name) - 11] = '\0';

	profile_no = -1;

	for(i = 0; profile[i].name[0] != '\0'; i++) {
		if (strcmp(profile[i].name, actual_name) == 0) {
			profile_no = i;
			break;		/* Chceme prvni,
					   jinak by to vzalo posledni */
		}
	}
}


void
on_b_new_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(w_main), FALSE);
	profile_no = -1;
	w_profile = create_w_profile();
	gtk_widget_show (w_profile);
}


void
on_b_chng_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	int sec, min, hour;
	
	gtk_widget_set_sensitive(GTK_WIDGET(w_main), FALSE);
	w_profile = create_w_profile ();
	
	/* Nastaveni hodnot */
	gtk_entry_set_text(GTK_ENTRY(e_prof_name), profile[profile_no].name);
	sec = profile[profile_no].time % 60;
	min = ((profile[profile_no].time - sec) / 60) % 60;
	hour = ((profile[profile_no].time - sec) / 60 - min) / 60;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_prof_sec), sec);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_prof_min), min);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_prof_hour), hour);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ch_event_start),
			profile[profile_no].cmd_start_en);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ch_event_stop),
			profile[profile_no].cmd_stop_en);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ch_event_done),
			profile[profile_no].cmd_done_en);

	gtk_entry_set_text(GTK_ENTRY(e_event_start),
			profile[profile_no].cmd_start);
	gtk_entry_set_text(GTK_ENTRY(e_event_stop),
			profile[profile_no].cmd_stop);
	gtk_entry_set_text(GTK_ENTRY(e_event_done),
			profile[profile_no].cmd_done);
	
	gtk_widget_show (w_profile);
}


void
on_b_del_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	int last;
	
	/* Najdem posledni, presunem jej na pozici
	 * soucasneho a zakoncime */

	/* Najdem posledni */
	last = 0;
	while (profile[last].name[0] != '\0')
		last++;
	last--;

	/* Nedovolime smazat posledni -- vzdy bude existovat aspon 1 */
	if(last == 0)
		return;
	
	
	/* Prekopirujem na misto smazane */
	memcpy(&(profile[profile_no]), &(profile[last]),
			sizeof(GTIMEOUT_PROFILE));
	
	/* smazeme posledni (original, ne kopii) */
	profile[last].name[0] = '\0';

	/* Posunem 'kurzor' */
	if (profile_no > 0)
		profile_no--;
	else
		profile_no = 0;
	
	/* Update + Serazeni */
	interface_update_profiles(profile, profile_no);
}


void
on_b_start_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	/* Set-Up main window*/
	gtk_widget_set_sensitive(GTK_WIDGET(f_profile), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_start), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_pause), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_stop), TRUE);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(b_pause), FALSE);

	if (profile[profile_no].cmd_start_en)
		event_execute(profile[profile_no].cmd_start);

	/* Enable Timeout 1 sec */
	time = profile[profile_no].time;
	gtk_progress_configure(GTK_PROGRESS(progressbar),
			time, 0, time);
	gtk_progress_set_value(GTK_PROGRESS(progressbar), time);
	on_timeout((gpointer) 1);
	time_out = gtk_timeout_add(1000, on_timeout, NULL);
	timeout_runing = 1;
}


void
on_b_pause_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b_pause))) {
		gtk_timeout_remove(time_out);
	} else {
		time_out = gtk_timeout_add(1000, on_timeout, NULL);
	}
}


void
on_b_stop_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	/* Set-Up main window*/
	gtk_widget_set_sensitive(GTK_WIDGET(f_profile), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_start), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_pause), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_stop), FALSE);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(b_pause), FALSE);

	/* Disable Timeout*/
	gtk_timeout_remove(time_out);

	timeout_runing = 0;
	
	if (profile[profile_no].cmd_stop_en && !user_data)
		event_execute(profile[profile_no].cmd_stop);
}

int on_timeout				(gpointer data)
{
	char str[128];
	long int hour, min, sec;	

	if (!data) {
		time--;
	}
	sec = time % 60;
	min = ((time - sec) / 60) % 60;
	hour = ((time - sec) / 60 - min) / 60;
	
	gtk_progress_set_value(GTK_PROGRESS(progressbar), time);
	sprintf(str, "%02ld:%02ld:%02ld", hour, min, sec);
	gtk_progress_set_format_string(GTK_PROGRESS(progressbar), str);
	
	/* If Timed out */
	if (time <= 0) {
		gtk_widget_set_sensitive(GTK_WIDGET(w_main), FALSE);
		gtk_progress_set_format_string(GTK_PROGRESS(progressbar),
				_("asova doasoval !"));
		on_b_stop_clicked(NULL, (gpointer) 1);

		if (profile[profile_no].cmd_done_en)
			event_execute(profile[profile_no].cmd_done);
		
		w_alarm = create_w_alarm (profile[profile_no].name);
		gtk_widget_show (w_alarm);

		return 0;
	}
	return 1;
}


void
on_b_about_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(w_main), FALSE);
	w_about = create_w_about ();
	gtk_widget_show (w_about);
}


void
on_b_exit_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(w_main);
}

int on_prof_changed() {
	return strlen(gtk_entry_get_text(GTK_ENTRY(e_prof_name))) > 0 &&
		(gtk_spin_button_get_value_as_int
			(GTK_SPIN_BUTTON(s_prof_sec)) +
		gtk_spin_button_get_value_as_int
			(GTK_SPIN_BUTTON(s_prof_min)) +
		gtk_spin_button_get_value_as_int
			(GTK_SPIN_BUTTON(s_prof_hour))) > 0;

		
}

void
on_e_prof_name_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(b_prof_ok),	on_prof_changed());
}


void
on_s_prof_hour_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(b_prof_ok),	on_prof_changed());
}


void
on_s_prof_min_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(b_prof_ok),	on_prof_changed());
}


void
on_s_prof_sec_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(b_prof_ok),	on_prof_changed());
}


void
on_b_prof_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	/* Pokud jde o novou polozku, tak ji vytvorime */
	if (profile_no == -1) {
		/* Najdem posledni profil a skoncime za nim */
		profile_no = 0;
		while(profile[profile_no].name[0] != '\0'
				&& profile_no <= PROFILE_MAX - 1)
			profile_no++;

		/* Zakonceni seznamu */
		profile[profile_no + 1].name[0] = '\0';
	}

	
	/* Ulozeni hodnot */
	strcpy(profile[profile_no].name,
			gtk_entry_get_text(GTK_ENTRY(e_prof_name)));
	
	profile[profile_no].time = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(s_prof_hour)) * 60 * 60;
	profile[profile_no].time += gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(s_prof_min)) * 60;
	profile[profile_no].time += gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(s_prof_sec));

	profile[profile_no].cmd_start_en = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(ch_event_start));
	strcpy(profile[profile_no].cmd_start,
			gtk_entry_get_text(GTK_ENTRY(e_event_start)));
	profile[profile_no].cmd_stop_en = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(ch_event_stop));
	strcpy(profile[profile_no].cmd_stop,
			gtk_entry_get_text(GTK_ENTRY(e_event_stop)));
	profile[profile_no].cmd_done_en = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(ch_event_done));
	strcpy(profile[profile_no].cmd_done,
			gtk_entry_get_text(GTK_ENTRY(e_event_done)));

	
	/* Likvidace okna */
	on_b_prof_cancel_clicked(NULL, NULL);
	
	/* Aktualizace vbr combo */
	interface_update_profiles(profile, profile_no);
}


void
on_b_prof_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	if (!user_data)
		gtk_widget_destroy(w_profile);

	gtk_widget_set_sensitive(GTK_WIDGET(w_main), TRUE);
}


void
on_b_about_close_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
	if (user_data)
		gtk_widget_destroy(GTK_WIDGET(user_data));
	gtk_widget_set_sensitive(GTK_WIDGET(w_main), TRUE);
}


void
on_b_alarm_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	if (!user_data)
		gtk_widget_destroy(w_alarm);

	gtk_widget_set_sensitive(GTK_WIDGET(w_main), TRUE);
}

