/******************************************************************************

		gTimeOut
  
	profile.c
  
	Josef Kufner 
 
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "profile.h"

#define CFG_FILE_NAME		".gtimeout"	/* slozi se v: 
						 * $HOME/CFG_FILE_NAME */

#define EN_CHAR(en)		((en)?'+':'-')

/* for debuging of save/load functions */
#define CFG_SAVE		1	/* normal: 1 */
#define CFG_LOAD		1	/* normal: 1 */


GTIMEOUT_PROFILE profile[PROFILE_MAX];

/*
 * Format souboru CFG_FILE_NAME:
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * >jmeno profilu, ktere konci novym radkem\n
 * \t	seconds\n
 * \t	+prikaz\n
 * \t	-vypnuty_prikaz\n
 * \t	+prikaz\n
 * \n
 * 
 * 
 */

char cfg_file_name_str[256] = "";
char *cfg_file_name() {
	strcpy(cfg_file_name_str, "");
	strcat(cfg_file_name_str, getenv("HOME"));
	strcat(cfg_file_name_str, "/");
	strcat(cfg_file_name_str, CFG_FILE_NAME);
	return cfg_file_name_str;	
}

int gtimeout_profile_save(GTIMEOUT_PROFILE *prof) {
#if CFG_SAVE
	FILE *f_cfg;
	int i;
	
	
	if ((f_cfg = fopen(cfg_file_name(), "wb")) == NULL) {
		return 1;
	};

	fprintf(f_cfg, "gTimeOut cofiguration file (cfg_ver=1)\n");
	for (i = 0; i < PROFILE_MAX && prof[i].name[0] != '\0'; i++) {
		fprintf(f_cfg, ">%s\n", prof[i].name);
		fprintf(f_cfg, "\t%ld\n", prof[i].time);
		fprintf(f_cfg, "\t%c%s\n", EN_CHAR(prof[i].cmd_start_en),
				prof[i].cmd_start);
		fprintf(f_cfg, "\t%c%s\n", EN_CHAR(prof[i].cmd_stop_en),
				prof[i].cmd_stop);
		fprintf(f_cfg, "\t%c%s\n", EN_CHAR(prof[i].cmd_done_en),
				prof[i].cmd_done);
		fprintf(f_cfg, "\n");
	}

	fclose(f_cfg);
#endif
	return 0;
}

static void load_to_eol (FILE *f_in, char *str, int max_len) {
	int i = 0, ch;
	
	str[0] = '\0';

	while(!feof(f_in)) {
		/* Nacteme znak a pokud to neni '\n' tak ho pridame do str */
		if (i < max_len - 1 && (ch = getc(f_in)) != '\n') {
			str[i] = ch;
			str[i+1] = '\0';
			i++;
		} else {
			/* a pokud jsme na konci radku tak koncime */
			str[i] = '\0';
			return;
		}
	}
}

static int load_event (FILE *f_in, int *en, char *str, int max_len) {
	char ch;
	
	if ((ch = getc(f_in)) != '\t') {
		/* Kdyz neni na zacatku '\t', nemame uz
		 * co delat*/
		ungetc(ch, f_in);
		return 0;
	}
	*en = (getc(f_in) == '+');
	load_to_eol(f_in, str, CMD_LEN);
	return 1;
}

int gtimeout_profile_load(GTIMEOUT_PROFILE *prof) {
#if CFG_LOAD
	FILE *f_cfg;
	int p = 0, ch;
	char str[256];		/* Sem nactem retezec, kde maji byt cisla */

	
	if ((f_cfg = fopen(cfg_file_name(), "rt")) != NULL ) {
		/* Load previously saved */
		for (p = 0; p < PROFILE_MAX && !feof(f_cfg); p++) {
			/* Nacteme textove */
			/* najdeme '>' na zacatku radku */
			while (!feof(f_cfg)) {
				if(getc(f_cfg) != '\n')
					continue;
				if(getc(f_cfg) != '>')
					continue;
				break;
			}

			/* nacteme jmeno profilu (muze obsahovat
			 * cokoli krom '\n') */
			load_to_eol(f_cfg, prof[p].name, NAME_LEN);

			/* nacteme cas v sekundach */
			if ((ch = getc(f_cfg)) == '\t') {
				load_to_eol(f_cfg, str, 256);
				sscanf(str, "%ld\n", &(prof[p].time));
					/* kdyby se to cetlo primo skrze
					 * fscanf, tak to nefunguje, protoze
					 * fscanf v tom udela bordel. */
			} else {
				ungetc(ch, f_cfg);
				continue;
			}

			/* nacteme eventy */
			/* Start */
			if (!load_event(f_cfg, &prof[p].cmd_start_en, 
					prof[p].cmd_start, CMD_LEN))
				continue;
			
			/* Stop */
			if(!load_event(f_cfg, &prof[p].cmd_stop_en, 
					prof[p].cmd_stop, CMD_LEN))
				continue;
			
			/* Done */
			if(!load_event(f_cfg, &prof[p].cmd_done_en, 
					prof[p].cmd_done, CMD_LEN))
				continue;
			
		}
		fclose(f_cfg);
	} else {
#endif
		/* Use Default */
		strcpy(prof[0].name, "Pizza");
		prof[0].time = 13 * 60 + 0;
		prof[0].cmd_done_en = 1;
		strcpy(prof[0].cmd_done, "beep -d 100 -f 660 -r 3");
	
		strcpy(prof[1].name, "Knedlky");
		prof[1].time = 7 * 60 + 30;
		prof[1].cmd_done_en = 1;
		strcpy(prof[1].cmd_done, "beep -d 100 -f 660 -r 3");
		
		strcpy(prof[2].name, "Jedna minuta");
		prof[2].time = 60;
		
		strcpy(prof[3].name, "A Test");
		prof[3].time = 5;
		prof[3].cmd_done_en = 1;
		strcpy(prof[3].cmd_done, "beep -d 100 -f 880 -r 3");
		
		prof[4].name[0] = '\0';
#if CFG_LOAD
	}
#endif
	return 0;
}

int gtimeout_profile_add(GTIMEOUT_PROFILE *prof) {
	printf("NOT Added!\n");
	return 0;
}


